@@ -1,12 +1,12 @@
 # --
 # Copyright (C) 2001-2021 OTRS AG, https://otrs.com/
-# Copyright (C) 2021-2022 Znuny GmbH, https://znuny.org/
+# Copyright (C) 2021 Znuny GmbH, https://znuny.org/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
 # did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
-## nofilter(TidyAll::Plugin::OTRS::Perl::Pod::SpellCheck)
+## nofilter(TidyAll::Plugin::Znuny::CodeStyle::STDERRCheck)
 
 package Kernel::Language;
 
@@ -407,7 +407,7 @@ DEPRECATED. Don't use this function any more, 'utf-8' is always the internal cha
 Returns the recommended charset for frontend (based on translation
 file or utf-8).
 
-    my $Charset = $LanguageObject->GetRecommendedCharset().
+    my $Charset = $LanguageObject->GetRecommendedCharset();
 
 =cut
 
@@ -421,7 +421,7 @@ sub GetRecommendedCharset {
 
 Returns an array of possible charsets (based on translation file).
 
-    my @Charsets = $LanguageObject->GetPossibleCharsets().
+    my @Charsets = $LanguageObject->GetPossibleCharsets();
 
 =cut
 
@@ -543,13 +543,30 @@ sub Time {
             $Day   = sprintf( "%02d",           $Day );
             $Month = sprintf( "%02d",           $Month );
         }
-        $ReturnString =~ s/\%T/$Time/g;
+
+        # if we have an dynamic field Date or DateTime, we want to add additional div's
+        # to use the css flex magic in the frontend
+        if ( $Param{DynamicFieldConfig} && $Param{DynamicFieldConfig}->{FieldType} =~ /(Date|DateTime)/ ) {
+            my ( $Date, $Time ) = split / - /, $ReturnString;
+            $ReturnString = '';
+            if ($Date) {
+                $ReturnString .= '<div>' . $Date . '</div>';
+            }
+
+            if ($Time) {
+                $ReturnString .= '<div>' . $Time . '</div>';
+            }
+        }
+
+        if ($Time) {
+            $ReturnString =~ s/\%T/$Time/g;
+        }
         $ReturnString =~ s/\%D/$Day/g;
         $ReturnString =~ s/\%M/$Month/g;
         $ReturnString =~ s/\%Y/$Year/g;
         $ReturnString =~ s{(\%A)}{defined $DayAbbr ? $Self->Translate($DayAbbr) : '';}egx;
-        $ReturnString
-            =~ s{(\%B)}{defined $MonthAbbr ? $Self->Translate($MonthAbbr) : '';}egx;
+        $ReturnString =~ s{(\%B)}{defined $MonthAbbr ? $Self->Translate($MonthAbbr) : '';}egx;
+
         return $ReturnString;
     }
 
