@@ -1254,7 +1254,10 @@ sub MaskAgentZoom {
 
     # get MoveQueuesStrg
     if ( $ConfigObject->Get('Ticket::Frontend::MoveType') =~ /^form$/i ) {
-        $MoveQueues{0}         = '- ' . $LayoutObject->{LanguageObject}->Translate('Move') . ' -';
+
+        my $MoveString = '- ' . $LayoutObject->{LanguageObject}->Translate('Move') . ' -';
+        $MoveQueues{0} = uc($MoveString);
+
         $Param{MoveQueuesStrg} = $LayoutObject->AgentQueueListOption(
             Name           => 'DestQueueID',
             Data           => \%MoveQueues,
@@ -1690,6 +1693,7 @@ sub MaskAgentZoom {
                                     # alias for ticket title, Title will be overwritten
                                     TicketTitle => $Ticket{Title},
                                     Value       => $Field->{Value},
+                                    ValueKey    => $Ticket{"DynamicField_$Field->{Name}"},
                                     Title       => $Field->{Title},
                                     Link        => $Field->{Link},
                                     LinkPreview => $Field->{LinkPreview},
@@ -1771,6 +1775,7 @@ sub MaskAgentZoom {
                         # alias for ticket title, Title will be overwritten
                         TicketTitle => $Ticket{Title},
                         Value       => $Field->{Value},
+                        ValueKey    => $Ticket{"DynamicField_$Field->{Name}"},
                         Title       => $Field->{Title},
                         Link        => $Field->{Link},
 
