@@ -890,20 +890,21 @@ sub Run {
                     }
                     elsif ( $Column eq 'Owner' || $Column eq 'Responsible' ) {
 
-                        $LayoutObject->Block(
-                            Name => 'ContentLargeTicketGenericHeaderColumnFilterLinkUserSearch',
-                            Data => {},
-                        );
+                      #136 - todo this autocomplete has been disabled for now
+                      #                         $LayoutObject->Block(
+                      #                             Name => 'ContentLargeTicketGenericHeaderColumnFilterLinkUserSearch',
+                      #                             Data => {},
+                      #                         );
 
-                        # send data to JS
-                        $LayoutObject->AddJSData(
-                            Key   => 'UserAutocomplete',
-                            Value => {
-                                'QueryDelay'          => 100,
-                                'MaxResultsDisplayed' => 20,
-                                'MinQueryLength'      => 2,
-                            },
-                        );
+                        #                         # send data to JS
+                        #                         $LayoutObject->AddJSData(
+                        #                             Key   => 'UserAutocomplete',
+                        #                             Value => {
+                        #                                 'QueryDelay'          => 100,
+                        #                                 'MaxResultsDisplayed' => 20,
+                        #                                 'MinQueryLength'      => 2,
+                        #                             },
+                        #                         );
                     }
 
                 }
@@ -1093,21 +1094,22 @@ sub Run {
                     );
                     if ( $Column eq 'CustomerUserID' ) {
 
-                        $LayoutObject->Block(
-                            Name =>
-                                'ContentLargeTicketGenericHeaderColumnFilterLinkCustomerUserSearch',
-                            Data => {},
-                        );
+                  #136 - todo this autocomplete has been disabled for now
+                  #                         $LayoutObject->Block(
+                  #                             Name =>
+                  #                                 'ContentLargeTicketGenericHeaderColumnFilterLinkCustomerUserSearch',
+                  #                             Data => {},
+                  #                         );
 
-                        # send data to JS
-                        $LayoutObject->AddJSData(
-                            Key   => 'CustomerUserAutocomplete',
-                            Value => {
-                                'QueryDelay'          => 100,
-                                'MaxResultsDisplayed' => 20,
-                                'MinQueryLength'      => 2,
-                            },
-                        );
+                        #                         # send data to JS
+                        #                         $LayoutObject->AddJSData(
+                        #                             Key   => 'CustomerUserAutocomplete',
+                        #                             Value => {
+                        #                                 'QueryDelay'          => 100,
+                        #                                 'MaxResultsDisplayed' => 20,
+                        #                                 'MinQueryLength'      => 2,
+                        #                             },
+                        #                         );
                     }
                 }
 
@@ -1643,6 +1645,11 @@ sub Run {
                     }
                 }
 
+                # add pill class
+                if ( $TicketColumn eq 'State' && IsStringWithData( $Article{StateID} ) ) {
+                    $CSSClass .= 'pill StateID-' . $Article{StateID};
+                }
+
                 $LayoutObject->Block(
                     Name => "RecordTicketColumn$BlockType",
                     Data => {
@@ -1909,7 +1916,7 @@ sub _InitialColumnFilter {
     my $ColumnFilterHTML = $LayoutObject->BuildSelection(
         Name        => 'ColumnFilter' . $Param{ColumnName},
         Data        => $Data,
-        Class       => $Class,
+        Class       => $Class . ' Modernize',
         Translation => $TranslationOption,
         SelectedID  => '',
     );
@@ -2032,15 +2039,10 @@ sub _ColumnFilterJSON {
 
         my %Values = %{ $Param{ColumnValues} };
 
-        # Keys must be link encoded for dynamic fields because they are added to URL during filtering
-        # and can contain characters like '&', ';', etc.
-        # See bug#14497 - https://bugs.otrs.org/show_bug.cgi?id=14497.
-        my $Encoding = ( $Param{ColumnName} =~ m/^DynamicField_/ ) ? 1 : 0;
-
         # Set possible values.
         for my $ValueKey ( sort { lc $Values{$a} cmp lc $Values{$b} } keys %Values ) {
             push @{$Data}, {
-                Key   => $Encoding ? $LayoutObject->LinkEncode($ValueKey) : $ValueKey,
+                Key   => $ValueKey,
                 Value => $Values{$ValueKey},
             };
         }
